@@ -22,6 +22,7 @@
 
 #include <ffi.h>
 
+#include <alloca.h>
 #include <alignof.h>
 #include <string.h>
 #include <assert.h>
@@ -44,8 +45,10 @@ SCM_SYMBOL (sym_uint32, "uint32");
 SCM_SYMBOL (sym_int32, "int32");
 SCM_SYMBOL (sym_uint64, "uint64");
 SCM_SYMBOL (sym_int64, "int64");
+SCM_SYMBOL (sym_short, "short");
 SCM_SYMBOL (sym_int, "int");
 SCM_SYMBOL (sym_long, "long");
+SCM_SYMBOL (sym_unsigned_short, "unsigned-short");
 SCM_SYMBOL (sym_unsigned_int, "unsigned-int");
 SCM_SYMBOL (sym_unsigned_long, "unsigned-long");
 SCM_SYMBOL (sym_size_t, "size_t");
@@ -606,6 +609,8 @@ make_cif (SCM return_type, SCM arg_types, const char *caller)
 	     + (nargs + n_struct_elts + 1)*sizeof(ffi_type));
 
   mem = scm_gc_malloc_pointerless (cif_len, "foreign");
+  /* ensure all the memory is initialized, even the holes */
+  memset (mem, 0, cif_len);
   cif = (ffi_cif *) mem;
 
   /* reuse cif_len to walk through the mem */
@@ -1039,6 +1044,30 @@ scm_init_foreign (void)
   scm_define (sym_uint64, scm_from_uint8 (SCM_FOREIGN_TYPE_UINT64));
   scm_define (sym_int64, scm_from_uint8 (SCM_FOREIGN_TYPE_INT64));
 
+  scm_define (sym_short,
+#if SIZEOF_SHORT == 8
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_INT64)
+#elif SIZEOF_SHORT == 4
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_INT32)
+#elif SIZEOF_SHORT == 2
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_INT16)
+#else
+# error unsupported sizeof (short)
+#endif
+	      );
+
+  scm_define (sym_unsigned_short,
+#if SIZEOF_SHORT == 8
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_UINT64)
+#elif SIZEOF_SHORT == 4
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_UINT32)
+#elif SIZEOF_SHORT == 2
+	      scm_from_uint8 (SCM_FOREIGN_TYPE_UINT16)
+#else
+# error unsupported sizeof (short)
+#endif
+	      );
+
   scm_define (sym_int,
 #if SIZEOF_INT == 8
 	      scm_from_uint8 (SCM_FOREIGN_TYPE_INT64)
