@@ -62,6 +62,8 @@
 #define R_OK 4
 #endif
 
+#include <stat-time.h>
+
 
 /* Loading a file, given an absolute filename.  */
 
@@ -167,8 +169,9 @@ SCM_DEFINE (scm_sys_library_dir, "%library-dir", 0,0,0,
 #ifdef SCM_SITE_DIR
 SCM_DEFINE (scm_sys_site_dir, "%site-dir", 0,0,0,
             (),
-	    "Return the directory where the Guile site files are installed.\n"
-	    "E.g., may return \"/usr/share/guile/site\".")
+	    "Return the directory where users should install Scheme code for use\n"
+            "with this version of Guile.\n\n"
+	    "E.g., may return \"/usr/share/guile/site/" SCM_EFFECTIVE_VERSION "\".")
 #define FUNC_NAME s_scm_sys_site_dir
 {
   return scm_from_locale_string (SCM_SITE_DIR);
@@ -176,6 +179,18 @@ SCM_DEFINE (scm_sys_site_dir, "%site-dir", 0,0,0,
 #undef FUNC_NAME
 #endif /* SCM_SITE_DIR */
 
+#ifdef SCM_GLOBAL_SITE_DIR
+SCM_DEFINE (scm_sys_global_site_dir, "%global-site-dir", 0,0,0,
+            (),
+	    "Return the directory where users should install Scheme code for use\n"
+            "with all versions of Guile.\n\n"
+	    "E.g., may return \"/usr/share/guile/site\".")
+#define FUNC_NAME s_scm_sys_global_site_dir
+{
+  return scm_from_locale_string (SCM_GLOBAL_SITE_DIR);
+}
+#undef FUNC_NAME
+#endif /* SCM_GLOBAL_SITE_DIR */
 
 
 
@@ -239,8 +254,9 @@ scm_init_load_path ()
   else if (env)
     path = scm_parse_path (scm_from_locale_string (env), path);
   else
-    path = scm_list_3 (scm_from_locale_string (SCM_LIBRARY_DIR),
+    path = scm_list_4 (scm_from_locale_string (SCM_LIBRARY_DIR),
                        scm_from_locale_string (SCM_SITE_DIR),
+                       scm_from_locale_string (SCM_GLOBAL_SITE_DIR),
                        scm_from_locale_string (SCM_PKGDATA_DIR));
 
   env = getenv ("GUILE_SYSTEM_COMPILED_PATH");
@@ -605,35 +621,49 @@ SCM_DEFINE (scm_sys_search_load_path, "%search-load-path", 1, 0, 0,
 #undef FUNC_NAME
 
 
+/* Return true if COMPILED_FILENAME is newer than source file
+   FULL_FILENAME, false otherwise.  Also return false if one of the
+   files cannot be stat'd.  */
 static int
 compiled_is_fresh (SCM full_filename, SCM compiled_filename)
 {
   char *source, *compiled;
   struct stat stat_source, stat_compiled;
-  int res;
+  int compiled_is_newer;
 
   source = scm_to_locale_string (full_filename);
   compiled = scm_to_locale_string (compiled_filename);
-    
+
   if (stat (source, &stat_source) == 0
-      && stat (compiled, &stat_compiled) == 0
-      && stat_source.st_mtime == stat_compiled.st_mtime) 
+      && stat (compiled, &stat_compiled) == 0)
     {
-      res = 1;
+      struct timespec source_mtime, compiled_mtime;
+
+      source_mtime = get_stat_mtime (&stat_source);
+      compiled_mtime = get_stat_mtime (&stat_compiled);
+
+      if (source_mtime.tv_sec < compiled_mtime.tv_sec
+	  || (source_mtime.tv_sec == compiled_mtime.tv_sec
+	      && source_mtime.tv_nsec <= compiled_mtime.tv_nsec))
+	compiled_is_newer = 1;
+      else
+	{
+	  compiled_is_newer = 0;
+	  scm_puts (";;; note: source file ", scm_current_error_port ());
+	  scm_puts (source, scm_current_error_port ());
+	  scm_puts ("\n;;;       newer than compiled ", scm_current_error_port ());
+	  scm_puts (compiled, scm_current_error_port ());
+	  scm_puts ("\n", scm_current_error_port ());
+	}
     }
   else
-    {
-      scm_puts (";;; note: source file ", scm_current_error_port ());
-      scm_puts (source, scm_current_error_port ());
-      scm_puts ("\n;;;       newer than compiled ", scm_current_error_port ());
-      scm_puts (compiled, scm_current_error_port ());
-      scm_puts ("\n", scm_current_error_port ());
-      res = 0;
-    }
+    /* At least one of the files isn't accessible.  */
+    compiled_is_newer = 0;
 
   free (source);
   free (compiled);
-  return res;
+
+  return compiled_is_newer;
 }
 
 SCM_KEYWORD (kw_env, "env");
@@ -761,6 +791,9 @@ SCM_DEFINE (scm_primitive_load_path, "primitive-load-path", 0, 0, 1,
     exception_on_not_found = SCM_BOOL_T;
 
   full_filename = scm_sys_search_load_path (filename);
+  if (scm_is_string (full_filename))
+    full_filename = scm_canonicalize_path (full_filename);
+
   compiled_filename =
     scm_search_path (*scm_loc_load_compiled_path,
 		     filename,
