@@ -121,7 +121,7 @@ extern char *ttyname();
 # include <xlocale.h>
 #endif
 
-#if HAVE_CRYPT_H
+#ifdef HAVE_CRYPT_H
 #  include <crypt.h>
 #endif
 
@@ -138,6 +138,7 @@ extern char *ttyname();
 #endif
 
 #include <sys/file.h>     /* from Gnulib */
+#include <nproc.h>
 
 /* Some Unix systems don't define these.  CPP hair is dangerous, but
    this seems safe enough... */
@@ -1755,7 +1756,7 @@ SCM_DEFINE (scm_sync, "sync", 0, 0, 0,
    multiple cpus.  So for now we don't bother with anything fancy, just
    ensure it works.  */
 
-#if HAVE_CRYPT
+#ifdef HAVE_CRYPT
 SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0,
             (SCM key, SCM salt),
 	    "Encrypt @var{key} using @var{salt} as the salt value to the\n"
@@ -1841,30 +1842,6 @@ SCM_DEFINE (scm_getlogin, "getlogin", 0, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_GETLOGIN */
 
-#if HAVE_CUSERID
-
-# if !HAVE_DECL_CUSERID
-extern char *cuserid (char *);
-# endif
-
-SCM_DEFINE (scm_cuserid, "cuserid", 0, 0, 0, 
-            (void),
-	    "Return a string containing a user name associated with the\n"
-	    "effective user id of the process.  Return @code{#f} if this\n"
-	    "information cannot be obtained.")
-#define FUNC_NAME s_scm_cuserid
-{
-  char buf[L_cuserid];
-  char * p;
-
-  p = cuserid (buf);
-  if (!p || !*p)
-    return SCM_BOOL_F;
-  return scm_from_locale_string (p);
-}
-#undef FUNC_NAME
-#endif /* HAVE_CUSERID */
-
 #if HAVE_GETPRIORITY
 SCM_DEFINE (scm_getpriority, "getpriority", 2, 0, 0, 
             (SCM which, SCM who),
@@ -1927,6 +1904,124 @@ SCM_DEFINE (scm_setpriority, "setpriority", 3, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_SETPRIORITY */
 
+#ifdef HAVE_SCHED_GETAFFINITY
+
+static SCM
+cpu_set_to_bitvector (const cpu_set_t *cs)
+{
+  SCM bv;
+  size_t cpu;
+
+  bv = scm_c_make_bitvector (sizeof (*cs), SCM_BOOL_F);
+
+  for (cpu = 0; cpu < sizeof (*cs); cpu++)
+    {
+      if (CPU_ISSET (cpu, cs))
+	/* XXX: This is inefficient but avoids code duplication.  */
+	scm_c_bitvector_set_x (bv, cpu, SCM_BOOL_T);
+    }
+
+  return bv;
+}
+
+SCM_DEFINE (scm_getaffinity, "getaffinity", 1, 0, 0,
+	    (SCM pid),
+	    "Return a bitvector representing the CPU affinity mask for\n"
+	    "process @var{pid}.  Each CPU the process has affinity with\n"
+	    "has its corresponding bit set in the returned bitvector.\n"
+	    "The number of bits set is a good estimate of how many CPUs\n"
+	    "Guile can use without stepping on other processes' toes.\n\n"
+	    "Currently this procedure is only defined on GNU variants\n"
+	    "(@pxref{CPU Affinity, @code{sched_getaffinity},, libc, The\n"
+	    "GNU C Library Reference Manual}).\n")
+#define FUNC_NAME s_scm_getaffinity
+{
+  int err;
+  cpu_set_t cs;
+
+  CPU_ZERO (&cs);
+  err = sched_getaffinity (scm_to_int (pid), sizeof (cs), &cs);
+  if (err)
+    SCM_SYSERROR;
+
+  return cpu_set_to_bitvector (&cs);
+}
+#undef FUNC_NAME
+
+#endif /* HAVE_SCHED_GETAFFINITY */
+
+#ifdef HAVE_SCHED_SETAFFINITY
+
+SCM_DEFINE (scm_setaffinity, "setaffinity", 2, 0, 0,
+	    (SCM pid, SCM mask),
+	    "Install the CPU affinity mask @var{mask}, a bitvector, for\n"
+	    "the process or thread with ID @var{pid}.  The return value\n"
+	    "is unspecified.\n\n"
+	    "Currently this procedure is only defined on GNU variants\n"
+	    "(@pxref{CPU Affinity, @code{sched_setaffinity},, libc, The\n"
+	    "GNU C Library Reference Manual}).\n")
+#define FUNC_NAME s_scm_setaffinity
+{
+  cpu_set_t cs;
+  scm_t_array_handle handle;
+  const scm_t_uint32 *c_mask;
+  size_t len, off, cpu;
+  ssize_t inc;
+  int err;
+
+  c_mask = scm_bitvector_elements (mask, &handle, &off, &len, &inc);
+
+  CPU_ZERO (&cs);
+  for (cpu = 0; cpu < len; cpu++)
+    {
+      size_t idx;
+
+      idx = cpu * inc + off;
+      if (c_mask[idx / 32] & (1UL << (idx % 32)))
+	CPU_SET (cpu, &cs);
+    }
+
+  err = sched_setaffinity (scm_to_int (pid), sizeof (cs), &cs);
+  if (err)
+    SCM_SYSERROR;
+
+  return SCM_UNSPECIFIED;
+}
+#undef FUNC_NAME
+
+#endif /* HAVE_SCHED_SETAFFINITY */
+
+SCM_DEFINE (scm_total_processor_count, "total-processor-count", 0, 0, 0,
+	    (void),
+	    "Return the total number of processors of the machine, which\n"
+	    "is guaranteed to be at least 1.  A ``processor'' here is a\n"
+	    "thread execution unit, which can be either:\n\n"
+	    "@itemize\n"
+	    "@item an execution core in a (possibly multi-core) chip, in a\n"
+	    "  (possibly multi- chip) module, in a single computer, or\n"
+	    "@item a thread execution unit inside a core in the case of\n"
+	    "  @dfn{hyper-threaded} CPUs.\n"
+	    "@end itemize\n\n"
+	    "Which of the two definitions is used, is unspecified.\n")
+#define FUNC_NAME s_scm_total_processor_count
+{
+  return scm_from_ulong (num_processors (NPROC_ALL));
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_current_processor_count, "current-processor-count", 0, 0, 0,
+	    (void),
+	    "Like @code{total-processor-count}, but return the number of\n"
+	    "processors available to the current process.  See\n"
+	    "@code{setaffinity} and @code{getaffinity} for more\n"
+	    "information.\n")
+#define FUNC_NAME s_scm_total_processor_count
+{
+  return scm_from_ulong (num_processors (NPROC_CURRENT));
+}
+#undef FUNC_NAME
+
+
 #if HAVE_GETPASS
 SCM_DEFINE (scm_getpass, "getpass", 1, 0, 0, 
             (SCM prompt),
@@ -2102,7 +2197,7 @@ SCM_DEFINE (scm_gethostname, "gethostname", 0, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_GETHOSTNAME */
 
-
+
 void
 scm_init_posix ()
 {
