@@ -38,8 +38,6 @@
 #include "libguile/validate.h"
 #include "libguile/socket.h"
 
-#include "libguile/iselect.h"
-
 #ifdef __MINGW32__
 #include "win32-socket.h"
 #endif
@@ -603,6 +601,16 @@ SCM_DEFINE (scm_setsockopt, "setsockopt", 4, 0, 0,
 	    "are defined (when provided by the system).  See @command{man\n"
 	    "ip} for what they mean.\n"
 	    "\n"
+	    "@defvar IP_MULTICAST_IF\n"
+            "This sets the source interface used by multicast traffic.\n"
+	    "@end defvar\n"
+	    "\n"
+	    "@defvar IP_MULTICAST_TTL\n"
+            "This sets the default TTL for multicast traffic. This defaults \n"
+            "to 1 and should be increased to allow traffic to pass beyond the\n"
+            "local network.\n"
+	    "@end defvar\n"
+	    "\n"
 	    "@defvar IP_ADD_MEMBERSHIP\n"
 	    "@defvarx IP_DROP_MEMBERSHIP\n"
 	    "These can be used only with @code{setsockopt}, not\n"
@@ -1275,30 +1283,16 @@ SCM_DEFINE (scm_accept, "accept", 1, 0, 0,
 	    "connection and will continue to accept new requests.")
 #define FUNC_NAME s_scm_accept
 {
-  int fd, selected;
+  int fd;
   int newfd;
   SCM address;
   SCM newsock;
-  SELECT_TYPE readfds, exceptfds;
   socklen_t addr_size = MAX_ADDR_SIZE;
   scm_t_max_sockaddr addr;
 
   sock = SCM_COERCE_OUTPORT (sock);
   SCM_VALIDATE_OPFPORT (1, sock);
   fd = SCM_FPORT_FDES (sock);
-
-  FD_ZERO (&readfds);
-  FD_ZERO (&exceptfds);
-  FD_SET (fd, &readfds);
-  FD_SET (fd, &exceptfds);
-
-  /* Block until something happens on FD, leaving guile mode while
-     waiting.  */
-  selected = scm_std_select (fd + 1, &readfds, NULL, &exceptfds,
-			     NULL);
-  if (selected < 0)
-    SCM_SYSERROR;
-
   newfd = accept (fd, (struct sockaddr *) &addr, &addr_size);
   if (newfd == -1)
     SCM_SYSERROR;
@@ -1765,6 +1759,14 @@ scm_init_socket ()
   scm_c_define ("IP_DROP_MEMBERSHIP", scm_from_int (IP_DROP_MEMBERSHIP));
 #endif
 
+#ifdef IP_MULTICAST_TTL 
+  scm_c_define ("IP_MULTICAST_TTL", scm_from_int ( IP_MULTICAST_TTL));
+#endif
+
+#ifdef IP_MULTICAST_IF 
+  scm_c_define ("IP_MULTICAST_IF", scm_from_int ( IP_MULTICAST_IF));
+#endif
+
   scm_add_feature ("socket");
 
 #include "libguile/socket.x"
