@@ -139,9 +139,18 @@ VM_NAME (SCM vm, SCM program, SCM *argv, int nargs)
     goto vm_error;
 
   vm_error_unbound:
-    /* At this point FINISH_ARGS should be a one-element list containing
-       the name of the unbound variable.  */
-    err_msg  = scm_from_locale_string ("VM: Unbound variable: ~s");
+    /* FINISH_ARGS should be the name of the unbound variable.  */
+    SYNC_ALL ();
+    err_msg = scm_from_locale_string ("Unbound variable: ~s");
+    scm_error_scm (scm_misc_error_key, program, err_msg,
+                   scm_list_1 (finish_args), SCM_BOOL_F);
+    goto vm_error;
+
+  vm_error_unbound_fluid:
+    SYNC_ALL ();
+    err_msg = scm_from_locale_string ("Unbound fluid: ~s");
+    scm_error_scm (scm_misc_error_key, program, err_msg,
+                   scm_list_1 (finish_args), SCM_BOOL_F);
     goto vm_error;
 
   vm_error_apply_to_non_list:
